@@ -8,11 +8,14 @@ Usage::
 """
 import sys
 import io
+import re
 import argparse
 import wordcloud as wc
 import numpy as np
 from PIL import Image
 
+from wordcloud import __version__
+
 
 class FileType(object):
     """Factory for creating file object types.
@@ -58,19 +61,26 @@ class FileType(object):
         return '%s(%s)' % (type(self).__name__, args_str)
 
 
-def main(args):
-    wordcloud = wc.WordCloud(
-        stopwords=args.stopwords, mask=args.mask,
-        width=args.width, height=args.height, font_path=args.font_path,
-        margin=args.margin, relative_scaling=args.relative_scaling,
-        color_func=args.color_func, background_color=args.background_color,
-        collocations=args.collocations)
-    wordcloud.generate(args.text)
+class RegExpAction(argparse.Action):
+     def __init__(self, option_strings, dest, **kwargs):
+         for option in option_strings:
+             try:
+                 re.compile(option)
+             except re.error as e:
+                 raise argparse.ArgumentError('Invalid regular expression: ' + str(e))
+         super(RegExpAction, self).__init__(option_strings, dest, **kwargs)
+
+     def __call__(self, parser, namespace, values, option_string=None):
+         setattr(namespace, self.dest, values)
+
+
+def main(args, text, imagefile):
+    wordcloud = wc.WordCloud(**args)
+    wordcloud.generate(text)
     image = wordcloud.to_image()
 
-    with args.imagefile:
-        out = args.imagefile if sys.version < '3' else args.imagefile.buffer
-        image.save(out, format='png')
+    with imagefile:
+        image.save(imagefile, format='png', optimize=True)
 
 
 def parse_args(arguments):
@@ -80,12 +90,15 @@ def parse_args(arguments):
     parser.add_argument(
         '--text', metavar='file', type=FileType(), default='-',
         help='specify file of words to build the word cloud (default: stdin)')
+    parser.add_argument(
+        '--regexp', metavar='regexp', default=None, action=RegExpAction,
+        help='override the regular expression defining what constitutes a word')
     parser.add_argument(
         '--stopwords', metavar='file', type=FileType(),
         help='specify file of stopwords (containing one word per line)'
              ' to remove from the given text after parsing')
     parser.add_argument(
-        '--imagefile', metavar='file', type=argparse.FileType('w'),
+        '--imagefile', metavar='file', type=argparse.FileType('wb'),
         default='-',
         help='file the completed PNG image should be written to'
              ' (default: stdout)')
@@ -93,10 +106,10 @@ def parse_args(arguments):
         '--fontfile', metavar='path', dest='font_path',
         help='path to font file you wish to use (default: DroidSansMono)')
     parser.add_argument(
-        '--mask', metavar='file', type=argparse.FileType(),
+        '--mask', metavar='file', type=argparse.FileType('rb'),
         help='mask to use for the image form')
     parser.add_argument(
-        '--colormask', metavar='file', type=argparse.FileType(),
+        '--colormask', metavar='file', type=argparse.FileType('rb'),
         help='color mask to use for image coloring')
     parser.add_argument(
         '--relative_scaling', type=float, default=0,
@@ -120,36 +133,42 @@ def parse_args(arguments):
         help='use given color as background color for the image -'
              ' accepts any value from PIL.ImageColor.getcolor')
     parser.add_argument(
-        '--no_collocations', action='store_true',
+        '--no_collocations', action='store_false', dest='collocations',
         help='do not add collocations (bigrams) to word cloud '
              '(default: add unigrams and bigrams)')
+    parser.add_argument('--version', action='version',
+        version='%(prog)s {version}'.format(version=__version__))
     args = parser.parse_args(arguments)
 
     if args.colormask and args.color:
         raise ValueError('specify either a color mask or a color function')
 
-    with args.text:
-        args.text = args.text.read()
+    args = vars(args)
 
-    if args.stopwords:
-        with args.stopwords:
-            args.stopwords = set(map(str.strip, args.stopwords.readlines()))
+    with args.pop('text') as f:
+        text = f.read()
 
-    if args.mask:
-        args.mask = np.array(Image.open(args.mask))
+    if args['stopwords']:
+        with args.pop('stopwords') as f:
+            args['stopwords'] = set(map(str.strip, f.readlines()))
+
+    if args['mask']:
+        mask = args.pop('mask')
+        args['mask'] = np.array(Image.open(mask))
 
     color_func = wc.random_color_func
-    if args.colormask:
-        image = np.array(Image.open(args.colormask))
+    colormask = args.pop('colormask')
+    color = args.pop('color')
+    if colormask:
+        image = np.array(Image.open(colormask))
         color_func = wc.ImageColorGenerator(image)
+    if color:
+        color_func = wc.get_single_color_func(color)
+    args['color_func'] = color_func
 
-    if args.color:
-        color_func = wc.get_single_color_func(args.color)
+    imagefile = args.pop('imagefile')
 
-    args.collocations = not args.no_collocations
-
-    args.color_func = color_func
-    return args
+    return args, text, imagefile
 
 if __name__ == '__main__':
-    main(parse_args(sys.argv[1:]))
+    main(*parse_args(sys.argv[1:]))
